@@ -26,6 +26,7 @@ if hasattr(xml,"use_pyxml"):
 from xml import xpath
 import xml.etree.ElementTree as ET
 import pexpect
+from calculate.lib.configparser import ConfigParser
 
 from calculate.lib.cl_lang import setLocalTranslate, getLazyLocalTranslate
 from calculate.lib.utils.colortext.palette import TextState
@@ -184,17 +185,19 @@ class Layman:
     def _add_to_laymanconf(self, rname, rurl, rpath):
         if not self.laymanconf:
             return
-        from calculate.lib.cl_template import iniParser
-        ini = iniParser(self.laymanconf)
-        if "rname" in ini.getAllSectionNames():
-            return
-        newrepo = {'auto-sync': 'Yes',
-                   'layman-type': 'git',
-                   'priority': '50',
-                   'sync-uri':rurl,
-                   'sync-type': 'laymansync',
-                   'location': rpath}
-        ini.setVar(rname, newrepo)
+        config = ConfigParser(strict=False)
+        config.read(self.laymanconf, encoding="utf-8")
+        if not config.has_section(rname):
+            config.add_section(rname)
+        for k,v in {'auto-sync': 'Yes',
+                    'layman-type': 'git',
+                    'priority': '50',
+                    'sync-uri':rurl,
+                    'sync-type': 'laymansync',
+                    'location': rpath}.items():
+            config.set(rname, k, v)
+        with open(self.laymanconf,'wb') as f:
+            config.write(f)
             
     def _add_to_installed(self, rname, rurl):
         """
@@ -325,9 +328,10 @@ class Git:
         return path.isdir(Git._gitDir(gitpath))
 
     def get_url(self, rpath, remote_name):
-        from calculate.lib.cl_template import iniParser
-        ini_parser = iniParser(path.join(Git._gitDir(rpath), "config"))
-        return ini_parser.getVar(u'remote "%s"'%remote_name, "url")
+        config = ConfigParser(strict=False)
+        config.read(path.join(Git._gitDir(rpath), "config"), encoding="utf-8")
+        return config.get('remote "%s"'%remote_name, "url",
+                          raw=True, fallback="")
 
     def cloneRepository(self, url, rpath, branch, cb_progress=None):
         """
