@@ -19,9 +19,14 @@ import sys
 from os import path
 from itertools import *
 from calculate.lib.datavars import (TableVariable,Variable,ReadonlyVariable,
-                                   VariableError)
+                                   VariableError, ReadonlyTableVariable,
+                                   FieldValue)
 import calculate.lib.cl_template as cl_template
 from calculate.lib.utils.files import listDirectory,process,PIPE,pathJoin
+from calculate.lib.utils.portage import isPkgInstalled
+from calculate.lib.utils.common import cmpVersion
+from itertools import dropwhile
+import re
 
 from calculate.lib.cl_lang import setLocalTranslate
 setLocalTranslate('cl_lib3',sys.modules[__name__])
@@ -310,6 +315,9 @@ class VariableClConfigProtectMask(ReadonlyVariable):
     type = "list"
 
     def get(self):
+        emerge_config = self.Get('cl_emerge_config')
+        if emerge_config and "CONFIG_PROTECT_MASK" in emerge_config:
+            return emerge_config['CONFIG_PROTECT_MASK'].encode('utf-8').split()
         displayEnv = process('/bin/bash',"-c","source /etc/profile;env",
                              stdout=PIPE)
         for line in displayEnv:
@@ -327,6 +335,9 @@ class VariableClConfigProtect(ReadonlyVariable):
     type = "list"
 
     def get(self):
+        emerge_config = self.Get('cl_emerge_config')
+        if emerge_config and "CONFIG_PROTECT" in emerge_config:
+            return emerge_config['CONFIG_PROTECT'].encode('utf-8').split()
         displayEnv = process('/bin/bash',"-c","source /etc/profile;env",
                              stdout=PIPE)
         for line in displayEnv:
@@ -358,6 +369,17 @@ class VariableClEbuildPhase(ReadonlyVariable):
     def get(self):
         return os.environ.get("EBUILD_PHASE","")
 
+class EmergeInfoData:
+    def __init__(self):
+        self.data = process("/usr/bin/emerge","--ask=n","--info",
+                       envdict=os.environ).read().split('\n')
+
+    def __iter__(self):
+        return iter(self.data)
+
+    def __str__(self):
+        return "emerge --info"
+
 class VariableClEmergeInfo(ReadonlyVariable):
     """
     Emerge --info cache
@@ -365,34 +387,87 @@ class VariableClEmergeInfo(ReadonlyVariable):
     type = "list"
 
     def get(self):
-        return filter(lambda x:x.startswith('PORTDIR_OVERLAY=') or \
-                               x.startswith('EMERGE') or \
-                               x.startswith('PKGDIR=') or \
-                               x.startswith('PORTDIR='),
-               process("/usr/bin/emerge","--ask=n","--info",
-                       envdict=os.environ).read().split('\n'))
+        return EmergeInfoData()
 
 class VariableClPkgdir(ReadonlyVariable):
     """
     PKGDIR
     """
     def get(self):
-        emergeInfo = self.Get('cl_emerge_info')
-        for line in filter(lambda x:x.startswith("PKGDIR="),
-                           emergeInfo):
-            return line.partition("=")[2].strip('\n"\'')
+        emerge_config = self.Get('cl_emerge_config')
+        if emerge_config and "PKGDIR" in emerge_config:
+            return emerge_config['PKGDIR'].encode('utf-8')
+        else:
+            emergeInfo = self.Get('cl_emerge_info')
+            for line in filter(lambda x:x.startswith("PKGDIR="),
+                               emergeInfo):
+                return line.partition("=")[2].strip('\n"\'')
         return ""
 
+class VariableClRepositoryData(ReadonlyTableVariable):
+    """
+    """
+    source = ["cl_repository_name", "cl_repository_location"]
+
+    def portage_ver_ge(self, ver):
+        portage_package = "sys-apps/portage"
+        portage_pkg_info = isPkgInstalled(portage_package)
+        if portage_pkg_info:
+            return cmpVersion(portage_pkg_info[0]['PV'], ver) >= 0
+        return False
+
+    def parse_repository_block(self, i):
+        text = "\n".join(i)
+        reBlock = re.compile("^(\w+)$\n\s+location: (.*)$", re.M)
+        return reBlock.findall(text)
+
+    def from_portdir_vars(self, info):
+        for line in filter(lambda x:x.startswith("PORTDIR="),
+                           info):
+            yield ["gentoo", line.partition("=")[2].strip('\n"\'')]
+
+        emergeInfo = self.Get('cl_emerge_info')
+        for line in filter(lambda x:x.startswith("PORTDIR_OVERLAY="),
+                    info):
+            for i in ([path.basename(x),x]
+                for x in line.partition("=")[2].strip('\n"\'').split(' ')
+                if x):
+                yield i
+
+    def get(self):
+        emerge_config = self.Get('cl_emerge_config')
+        if emerge_config and "PKGDIR" in emerge_config:
+            return [[repo.name.encode('UTF-8'),repo.location.encode('UTF-8')]
+                for repo in emerge_config.repositories]
+        else:
+            info = self.Get("cl_emerge_info")
+            if self.portage_ver_ge("2.2.18"):
+                return self.parse_repository_block(
+                    dropwhile(lambda x:not x.startswith("Repositories:"), info))
+            return list(self.from_portdir_vars(info))
+
+class VariableClRepositoryName(FieldValue,ReadonlyVariable):
+    """
+    """
+    type = "list"
+    source_variable = "cl_repository_data"
+    column = 0
+
+class VariableClRepositoryLocation(FieldValue,ReadonlyVariable):
+    """
+    """
+    type = "list"
+    source_variable = "cl_repository_data"
+    column = 1
+
 class VariableClPortdir(ReadonlyVariable):
     """
     PORTDIR
     """
     def get(self):
-        emergeInfo = self.Get('cl_emerge_info')
-        for line in filter(lambda x:x.startswith("PORTDIR="),
-                           emergeInfo):
-            return line.partition("=")[2].strip('\n"\'')
-        return ""
+        return self.Select('cl_repository_location',
+                           where='cl_repository_name',
+                           eq="gentoo", limit=1) or ""
 
 class VariableClPortdirOverlay(ReadonlyVariable):
     """
@@ -401,12 +476,26 @@ class VariableClPortdirOverlay(ReadonlyVariable):
     type = "list"
 
     def get(self):
-        emergeInfo = self.Get('cl_emerge_info')
-        for line in filter(lambda x:x.startswith("PORTDIR_OVERLAY="),
-                    emergeInfo):
-            return filter(None,
-                   line.partition("=")[2].strip('\n"\'').split(' '))
-        return []
+        return self.Select('cl_repository_location',
+                           where='cl_repository_name',
+                           ne="gentoo") or []
+
+class VariableClEmergeConfig(ReadonlyVariable):
+    """
+    Emerge config object
+    """
+    type = "object"
+
+    def get(self):
+        try:
+            from portage.package.ebuild.config import config
+            config.__str__ = lambda self: "EmergeConfig"
+            c = config()
+            if c.repositories.repoLocationList() and c["LINGUAS"]:
+                return c
+        except:
+            pass
+        return ""
 
 class VariableClEmergeDefaultOpts(ReadonlyVariable):
     """
@@ -414,15 +503,20 @@ class VariableClEmergeDefaultOpts(ReadonlyVariable):
     """
 
     def get(self):
-        emergeInfo = self.Get('cl_emerge_info')
-        for line in filter(lambda x:x.startswith("EMERGE_DEFAULT_OPTS="),
-                           emergeInfo):
-            key, op, value = line.partition('=')
-            for quote in ("'",'"'):
-                if value.startswith(quote):
-                    value = value[1:-1]
-            return value.replace(r'\"','"')
-        return []
+        emerge_config = self.Get('cl_emerge_config')
+        if emerge_config:
+            return emerge_config['EMERGE_DEFAULT_OPTS'].encode('UTF-8')
+        else:
+            emergeInfo = self.Get('cl_emerge_info')
+            for line in filter(lambda x:x.startswith("EMERGE_DEFAULT_OPTS="),
+                               emergeInfo):
+                key, op, value = line.partition('=')
+                for quote in ("'",'"'):
+                    if value.startswith(quote):
+                        value = value[1:-1]
+                return value.replace(r'\"','"')
+        return ""
+
 
 class VariableClTemplatesLocate(Variable):
     """
